//I/O stuff
local configplayer
local changesoundsch

local function SaveCVar(cvarname, path)
	local file = io.openlocal(path, "w+")
	local cvar = CV_FindVar(cvarname)
	file:write(cvar.value)
	file:close()
end

local function LoadCVar(cvarname, path)
	local file = io.openlocal(path)
	local cvar = CV_FindVar(cvarname)
	if file
		local num = file:read("*n")
		CV_Set(cvar, num)
		file:close()
	end
end

local function SavePVar(path, toinsert)
	local file = io.openlocal(path, "w+")
	if type(toinsert) == "string"
		file:write(toinsert)
	else
		file:write(tostring(toinsert))
	end
	file:close()
end

local function L_DecimalFixed(str)
	if str == nil return nil end
	local dec_offset = string.find(str,'%.')
	if dec_offset == nil
		return (tonumber(str) or 0)*FRACUNIT
	end
	
	local start, finish, _ = string.find(str,'%.%d+')
	local first
	if start == 1 then
		first = string.match(str, '%d+')
	else
		first = tonumber(string.match(str, '%d+')..string.match(str, '%d+', start))
	end
	local second = 10^(finish - start)
	
	return FixedDiv(first*FRACUNIT, second*FRACUNIT)
end

local function L_FixedDecimal(str,maxdecimal)
	if str == nil or tostring(str) == nil
		error("First argument is not a valid fixed value", 2)
		return
	end
	local number = tonumber(str)
	maxdecimal = ($ != nil) and $ or 3
	if tonumber(str) == 0 return '0' end
	local polarity = abs(number)/number
	local str_polarity = (polarity < 0) and '-' or ''
	local str_whole = tostring(abs(number/FRACUNIT))
	if maxdecimal == 0
		return str_polarity..str_whole
	end
	local decimal = number%FRACUNIT
	decimal = FRACUNIT + $
	decimal = FixedMul($,FRACUNIT*10^maxdecimal)
	decimal = $>>FRACBITS
	local str_decimal = string.sub(decimal,2)
	return str_polarity..str_whole..'.'..str_decimal
end

//Aura state definition
freeslot("S_SUPERAURA", "S_AURAOVERLAY", "S_SUPERAURA2", "S_AURAOVERLAY2")
states[S_SUPERAURA] = {
	sprite = SPR_SSJA,
	frame = A|FF_FULLBRIGHT,
	var1 = 1,
	nextstate = S_SUPERAURA
}

states[S_AURAOVERLAY] = {
	sprite = SPR_SPAU,
	frame = A|FF_FULLBRIGHT,
	nextstate = S_AURAOVERLAY
}

states[S_SUPERAURA2] = {
	sprite = SPR_SSJA,
	frame = A|FF_FULLBRIGHT,
	var1 = 1,
	nextstate = S_SUPERAURA2
}

states[S_AURAOVERLAY2] = {
	sprite = SPR_SPAU,
	frame = A|FF_FULLBRIGHT,
	nextstate = S_AURAOVERLAY2
}

freeslot("S_GLOWOL")
states[S_GLOWOL] = {SPR_NULL, A|FF_FULLBRIGHT, 2, nil, 1, 0, S_NULL}

//Aura settings
CV_RegisterVar({
	name = "sfaura_sounds",
	defaultvalue = "On",
	flags = CV_SHOWMODIF|CV_CALL,
	PossibleValue = CV_OnOff,
	func = function(var)
		for player in players.iterate
			if var.value == 0
			and (player.auramo and player.auramo.valid) then
				S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
			end
		end
	end
})

CV_RegisterVar({
	name = "sfaura_allowsounds",
	defaultvalue = "On",
	flags = CV_SHOWMODIF|CV_NETVAR|CV_CALL,
	PossibleValue = CV_OnOff,
	func = function(var)
		for player in players.iterate
			if var.value == 0
			and (player.auramo and player.auramo.valid) then
				S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
				player.auramo.passfxtimer = 0
			end
		end
	end
})

local function SFA_XScale(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.auraxscale = L_DecimalFixed(arg)
				SavePVar("client/SuperFormAuras/sfaura_xscale.dat", arg)
				CONS_Printf(player, "Aura X size is now "..L_FixedDecimal(player.auraxscale))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_xscale <value> (0-5, decimals allowed): Sets the aura's horizontal size (width).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.auraxscale))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_YScale(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.aurayscale = L_DecimalFixed(arg)
				SavePVar("client/SuperFormAuras/sfaura_yscale.dat", arg)
				CONS_Printf(player, "Aura Y size is now "..L_FixedDecimal(player.aurayscale))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_yscale <value> (0-5, decimals allowed): Sets the aura's vertical size (height).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.aurayscale))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_XScale2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.auraxscale2 = L_DecimalFixed(arg)
				SavePVar("client/SuperFormAuras/sfaura_xscale2.dat", arg)
				CONS_Printf(player, "Second aura X size is now "..L_FixedDecimal(player.auraxscale2))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_xscale2 <value> (0-5, decimals allowed): Sets the second aura's horizontal size (width).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.auraxscale2))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_YScale2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tostring(arg)
			if L_DecimalFixed(arg) > 0 and L_DecimalFixed(arg) <= 5*FRACUNIT then
				player.aurayscale2 = L_DecimalFixed(arg)
				SavePVar("client/SuperFormAuras/sfaura_yscale2.dat", arg)
				CONS_Printf(player, "Second aura Y size is now "..L_FixedDecimal(player.aurayscale2))
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_yscale2 <value> (0-5, decimals allowed): Sets the second aura's vertical size (height).")
			CONS_Printf(player, "Current: "..L_FixedDecimal(player.aurayscale2))
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Conditions(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "never" then
				player.auraconditions = 0
				SavePVar("client/SuperFormAuras/sfaura_conditions.dat", player.auraconditions)
				CONS_Printf(player, "Aura will now be always disabled.")
			elseif arg == "1" or arg == "super" then
				player.auraconditions = 1
				SavePVar("client/SuperFormAuras/sfaura_conditions.dat", player.auraconditions)
				CONS_Printf(player, "Aura will now only trigger when you go Super.")
			elseif arg == "2" or arg == "always" then
				player.auraconditions = 2
				SavePVar("client/SuperFormAuras/sfaura_conditions.dat", player.auraconditions)
				CONS_Printf(player, "Aura will now be always enabled.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_conditions <never / super / always>: Sets the conditions that must be met in order to trigger the aura.")
			if player.auraconditions == 0 then
				CONS_Printf(player, "Current: Never")
			elseif player.auraconditions == 1 then
				CONS_Printf(player, "Current: Super Only")
			else
				CONS_Printf(player, "Current: Always")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Color(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if R_GetColorByName(arg) != SKINCOLOR_NONE then
				player.auracolor = R_GetColorByName(arg)
				SavePVar("client/SuperFormAuras/sfaura_color.dat", player.auracolor)
				CONS_Printf(player, "Aura color has been set to "..R_GetNameByColor(R_GetColorByName(arg)))
			elseif arg == "player" then
				player.auracolor = "player"
				SavePVar("client/SuperFormAuras/sfaura_color.dat", player.auracolor)
				CONS_Printf(player, "Aura color will now be the same as the player's color.")
			elseif arg == "opposite" then
				player.auracolor = "opposite"
				SavePVar("client/SuperFormAuras/sfaura_color.dat", player.auracolor)
				CONS_Printf(player, "Aura color will now be the opposite of the player's color.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_color <skincolor number/name>: Sets the aura's color.")
			if type(player.auracolor) == "number" then
				CONS_Printf(player, "Current aura color is "..R_GetNameByColor(player.auracolor))
			else
				CONS_Printf(player, "Current aura color is: "..player.auracolor)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Color2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if R_GetColorByName(arg) != SKINCOLOR_NONE then
				player.auracolor2 = R_GetColorByName(arg)
				SavePVar("client/SuperFormAuras/sfaura_color2.dat", player.auracolor2)
				CONS_Printf(player, "Second aura color has been set to "..R_GetNameByColor(R_GetColorByName(arg)))
			elseif arg == "player" then
				player.auracolor2 = "player"
				SavePVar("client/SuperFormAuras/sfaura_color2.dat", player.auracolor2)
				CONS_Printf(player, "Second aura color will now be the same as the player's color.")
			elseif arg == "opposite" then
				player.auracolor2 = "opposite"
				SavePVar("client/SuperFormAuras/sfaura_color2.dat", player.auracolor2)
				CONS_Printf(player, "Second aura color will now be the opposite of the player's color.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_color2 <skincolor number/name>: Sets the second aura's color.")
			if type(player.auracolor2) == "number" then
				CONS_Printf(player, "Current second aura color is "..R_GetNameByColor(player.auracolor2))
			else
				CONS_Printf(player, "Current second aura color is: "..player.auracolor2)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_SoundScheme(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_SoundSchemes[arg] != nil
				if (player.auramo and player.auramo.valid) then
					S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
					player.auramo.passfxtimer = 0
				end
				player.aurasoundsch = arg
				SavePVar("client/SuperFormAuras/sfaura_soundscheme.dat", arg)
				if changesoundsch then
					S_StartSound(player.auramo, SFA_SoundSchemes[player.aurasoundsch].transform)
				end
				CONS_Printf(player, "Aura sound scheme is now "..SFA_SoundSchemes[arg].name)
			elseif arg == "list"
				for k, v in pairs(SFA_SoundSchemes)
					CONS_Printf(player, k.." - "..SFA_SoundSchemes[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_soundscheme <id>: Sets the sound scheme to be used by the aura.")
			CONS_Printf(player, "Current: "..SFA_SoundSchemes[player.aurasoundsch].name)
			CONS_Printf(player, "Type \"sfaura_soundscheme list\" to see all the available IDs for sound schemes.")
			
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Sprite(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_AuraStyles[arg] != nil
				if (player.auramo and player.auramo.valid) then
					player.auramo.frame = A|($ & ~FF_FRAMEMASK)
					player.auramo.nextframe = 0
					player.auramo.sprite = SFA_AuraStyles[arg].sprite
				end
				player.aurastyle = arg
				SavePVar("client/SuperFormAuras/sfaura_sprite.dat", arg)
				CONS_Printf(player, "Aura sprite style is now "..SFA_AuraStyles[arg].name)
			elseif arg == "list"
				for k, v in pairs(SFA_AuraStyles)
					CONS_Printf(player, k.." - "..SFA_AuraStyles[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_sprite <id>: Sets the sprite style of the aura.")
			CONS_Printf(player, "Current: "..SFA_AuraStyles[player.aurastyle].name)
			CONS_Printf(player, "Type \"sfaura_sprite list\" to see all the available IDs for aura styles.")
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Sprite2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_AuraStyles[arg] != nil
				if (player.auramo2 and player.auramo2.valid) then
					player.auramo2.frame = A|($ & ~FF_FRAMEMASK)
					player.auramo2.nextframe = 0
					player.auramo2.sprite = SFA_AuraStyles[arg].sprite
				end
				player.aurastyle2 = arg
				SavePVar("client/SuperFormAuras/sfaura_sprite2.dat", arg)
				CONS_Printf(player, "Second aura sprite style is now "..SFA_AuraStyles[arg].name)
			elseif arg == "list"
				for k, v in pairs(SFA_AuraStyles)
					CONS_Printf(player, k.." - "..SFA_AuraStyles[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_sprite2 <id>: Sets the sprite style of the second aura.")
			CONS_Printf(player, "Current: "..SFA_AuraStyles[player.aurastyle2].name)
			CONS_Printf(player, "Type \"sfaura_sprite2 list\" to see all the available IDs for aura styles.")
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Opacity(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tonumber(arg)
			if arg >= 0 and arg <= 10 then
				player.auraopacity = arg
				if player.auramo and player.auramo.valid then
					if player.auraopacity == 0 then
						player.auramo.flags2 = $|MF2_DONTDRAW
					elseif player.auraopacity > 0 and player.auraopacity < 10
						player.auramo.flags2 = $ & ~MF2_DONTDRAW
						player.auramo.frame = ($ & ~FF_TRANSMASK)|((10-player.auraopacity)<<FF_TRANSSHIFT)
					else
						player.auramo.frame = $ & ~FF_TRANSMASK
						player.auramo.flags2 = $ & ~MF2_DONTDRAW
					end
				end
				SavePVar("client/SuperFormAuras/sfaura_opacity.dat", arg)
				if player.auraopacity > 0 then
					CONS_Printf(player, "Aura opacity is now "..player.auraopacity.."0%")
				else
					CONS_Printf(player, "Aura opacity is now "..player.auraopacity)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_opacity <value> (0-10): Sets the aura's opacity.")
			if player.auraopacity > 0 then
				CONS_Printf(player, "Current: "..player.auraopacity.."0%")
			else
				CONS_Printf(player, "Current: "..player.auraopacity)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Opacity2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = tonumber(arg)
			if arg >= 0 and arg <= 10 then
				player.auraopacity2 = arg
				if player.auramo2 and player.auramo2.valid then
					if player.auraopacity2 == 0 then
						player.auramo2.flags2 = $|MF2_DONTDRAW
					elseif player.auraopacity2 > 0 and player.auraopacity2 < 10
						player.auramo2.flags2 = $ & ~MF2_DONTDRAW
						player.auramo2.frame = ($ & ~FF_TRANSMASK)|((10-player.auraopacity2)<<FF_TRANSSHIFT)
					else
						player.auramo2.frame = $ & ~FF_TRANSMASK
						player.auramo2.flags2 = $ & ~MF2_DONTDRAW
					end
				end
				SavePVar("client/SuperFormAuras/sfaura_opacity2.dat", arg)
				if player.auraopacity2 > 0 then
					CONS_Printf(player, "Second aura opacity is now "..player.auraopacity2.."0%")
				else
					CONS_Printf(player, "Second aura opacity is now "..player.auraopacity2)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_opacity2 <value> (0-10): Sets the second aura's opacity.")
			if player.auraopacity > 0 then
				CONS_Printf(player, "Current: "..player.auraopacity2.."0%")
			else
				CONS_Printf(player, "Current: "..player.auraopacity2)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_DrawOrder(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "front" then
				player.auradraworder = 0
				SavePVar("client/SuperFormAuras/sfaura_draworder.dat", arg)
				CONS_Printf(player, "The aura will now be shown in front of the player.")
			elseif arg == "1" or arg == "behind" then
				player.auradraworder = 1
				SavePVar("client/SuperFormAuras/sfaura_draworder.dat", arg)
				CONS_Printf(player, "The aura will now be shown behind the player.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_draworder <front / behind>: Sets if the aura should be shown behind or in front of the player.")
			if player.auradraworder == 0 then
				CONS_Printf(player, "Current: Front")
			else
				CONS_Printf(player, "Current: Behind")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_DrawOrder2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "front" then
				player.auradraworder2 = 0
				SavePVar("client/SuperFormAuras/sfaura_draworder2.dat", arg)
				CONS_Printf(player, "The second aura will now be shown in front of the player.")
			elseif arg == "1" or arg == "behind" then
				player.auradraworder2 = 1
				SavePVar("client/SuperFormAuras/sfaura_draworder2.dat", arg)
				CONS_Printf(player, "The second aura will now be shown behind the player.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_draworder2 <front / behind>: Sets if the second aura should be shown behind or in front of the player.")
			if player.auradraworder2 == 0 then
				CONS_Printf(player, "Current: Front")
			else
				CONS_Printf(player, "Current: Behind")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_BlendMode(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "none" or arg == "normal" then
				player.aurablend = AST_TRANSLUCENT
				SavePVar("client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a normal blend mode.")
			elseif arg == "1" or arg == "add" then
				player.aurablend = AST_ADD
				SavePVar("client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have an additive blend mode.")
			elseif arg == "2" or arg == "subtract" then
				player.aurablend = AST_SUBTRACT
				SavePVar("client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a subtractive blend mode.")
			elseif arg == "3" or arg == "difference" then
				player.aurablend = AST_REVERSESUBTRACT
				SavePVar("client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a difference blend mode.")
			elseif arg == "4" or arg == "multiply" then
				player.aurablend = AST_MODULATE
				SavePVar("client/SuperFormAuras/sfaura_blendmode.dat", arg)
				CONS_Printf(player, "The aura will now have a multiplicative blend mode.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_blendmode <normal / add / subtract / difference / multiply>: Sets the blending mode of the aura.")
			if player.aurablend == AST_TRANSLUCENT then
				CONS_Printf(player, "Current: Normal")
			elseif player.aurablend == AST_ADD then
				CONS_Printf(player, "Current: Additive")
			elseif player.aurablend == AST_SUBTRACT then
				CONS_Printf(player, "Current: Subtractive")
			elseif player.aurablend == AST_REVERSESUBTRACT then
				CONS_Printf(player, "Current: Difference")
			elseif player.aurablend == AST_MODULATE then
				CONS_Printf(player, "Current: Multiplicative")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_BlendMode2(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "none" or arg == "normal" then
				player.aurablend2 = AST_TRANSLUCENT
				SavePVar("client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a normal blend mode.")
			elseif arg == "1" or arg == "add" then
				player.aurablend2 = AST_ADD
				SavePVar("client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have an additive blend mode.")
			elseif arg == "2" or arg == "subtract" then
				player.aurablend2 = AST_SUBTRACT
				SavePVar("client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a subtractive blend mode.")
			elseif arg == "3" or arg == "difference" then
				player.aurablend2 = AST_REVERSESUBTRACT
				SavePVar("client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a difference blend mode.")
			elseif arg == "4" or arg == "multiply" then
				player.aurablend2 = AST_MODULATE
				SavePVar("client/SuperFormAuras/sfaura_blendmode2.dat", arg)
				CONS_Printf(player, "The second aura will now have a multiplicative blend mode.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_blendmode2 <normal / add / subtract / difference / multiply>: Sets the blending mode of the second aura.")
			if player.aurablend2 == AST_TRANSLUCENT then
				CONS_Printf(player, "Current: Normal")
			elseif player.aurablend2 == AST_ADD then
				CONS_Printf(player, "Current: Additive")
			elseif player.aurablend2 == AST_SUBTRACT then
				CONS_Printf(player, "Current: Subtractive")
			elseif player.aurablend2 == AST_REVERSESUBTRACT then
				CONS_Printf(player, "Current: Difference")
			elseif player.aurablend2 == AST_MODULATE then
				CONS_Printf(player, "Current: Multiplicative")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_Outline(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "1" or arg == "on" then
				player.glowoutline = true
				SavePVar("client/SuperFormAuras/sfaura_outline.dat", arg)
				CONS_Printf(player, "The player's sprites will now have a glowing outline.")
			elseif arg == "0" or arg == "off" then
				player.glowoutline = false
				SavePVar("client/SuperFormAuras/sfaura_outline.dat", arg)
				CONS_Printf(player, "The player's sprites will no longer have a glowing outline.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_outline <on / off>: Sets if the player's sprites should have a glowing outline.")
			if player.glowoutline then
				CONS_Printf(player, "Current: Enabled")
			else
				CONS_Printf(player, "Current: Disabled")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_OutlineColor(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if R_GetColorByName(arg) != SKINCOLOR_NONE then
				player.outlinecolor = R_GetColorByName(arg)
				SavePVar("client/SuperFormAuras/sfaura_outlinecolor.dat", player.outlinecolor)
				CONS_Printf(player, "Outline color has been set to "..R_GetNameByColor(R_GetColorByName(arg)))
			elseif arg == "player" then
				player.outlinecolor = "player"
				SavePVar("client/SuperFormAuras/sfaura_outlinecolor.dat", player.outlinecolor)
				CONS_Printf(player, "Outline color will now be the same as the player's color.")
			elseif arg == "opposite" then
				player.outlinecolor = "opposite"
				SavePVar("client/SuperFormAuras/sfaura_outlinecolor.dat", player.outlinecolor)
				CONS_Printf(player, "Outline color will now be the opposite of the player's color.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_outlinecolor <skincolor number/name>: Sets the color of the sprite's glowing outline.")
			if type(player.outlinecolor) == "number" then
				CONS_Printf(player, "Current outline color is "..R_GetNameByColor(player.outlinecolor))
			else
				CONS_Printf(player, "Current outline color is: "..player.outlinecolor)
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

local function SFA_OutlineBlend(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if arg == "0" or arg == "add" then
				player.outlineblend = AST_ADD
				SavePVar("client/SuperFormAuras/sfaura_outlineblend.dat", arg)
				CONS_Printf(player, "The sprite outline will now have an additive blend mode.")
			elseif arg == "1" or arg == "subtract" then
				player.outlineblend = AST_SUBTRACT
				SavePVar("client/SuperFormAuras/sfaura_outlineblend.dat", arg)
				CONS_Printf(player, "The sprite outline will now have a subtractive blend mode.")
			elseif arg == "2" or arg == "multiply" then
				player.outlineblend = AST_MODULATE
				SavePVar("client/SuperFormAuras/sfaura_outlineblend.dat", arg)
				CONS_Printf(player, "The sprite outline will now have a multiplicative blend mode.")
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_outlineblend <add / subtract / multiply>: Sets the blending mode of the sprite outline.")
			if player.outlineblend == AST_ADD then
				CONS_Printf(player, "Current: Additive")
			elseif player.outlineblend == AST_SUBTRACT then
				CONS_Printf(player, "Current: Subtractive")
			elseif player.outlineblend == AST_MODULATE then
				CONS_Printf(player, "Current: Multiplicative")
			end
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end

COM_AddCommand("sfaura_xscale", SFA_XScale)
COM_AddCommand("sfaura_yscale", SFA_YScale)
COM_AddCommand("sfaura_xscale2", SFA_XScale2)
COM_AddCommand("sfaura_yscale2", SFA_YScale2)
COM_AddCommand("sfaura_conditions", SFA_Conditions)
COM_AddCommand("sfaura_color", SFA_Color)
COM_AddCommand("sfaura_color2", SFA_Color2)
COM_AddCommand("sfaura_soundscheme", SFA_SoundScheme)
COM_AddCommand("sfaura_sprite", SFA_Sprite)
COM_AddCommand("sfaura_sprite2", SFA_Sprite2)
COM_AddCommand("sfaura_opacity", SFA_Opacity)
COM_AddCommand("sfaura_opacity2", SFA_Opacity2)
COM_AddCommand("sfaura_draworder", SFA_DrawOrder)
COM_AddCommand("sfaura_draworder2", SFA_DrawOrder2)
COM_AddCommand("sfaura_blendmode", SFA_BlendMode)
COM_AddCommand("sfaura_blendmode2", SFA_BlendMode2)
COM_AddCommand("sfaura_outline", SFA_Outline)
COM_AddCommand("sfaura_outlinecolor", SFA_OutlineColor)
COM_AddCommand("sfaura_outlineblend", SFA_OutlineBlend)

COM_AddCommand("sfaura_preset", function(player, arg)
	if (player.mo and player.mo.valid) then
		if arg then
			arg = arg:lower()
			if SFA_Presets[arg] != nil then
				if SFA_Presets[arg].sfaura_xscale != nil then
					SFA_XScale(player, tostring(SFA_Presets[arg].sfaura_xscale))
				end
				if SFA_Presets[arg].sfaura_yscale != nil then
					SFA_YScale(player, tostring(SFA_Presets[arg].sfaura_yscale))
				end
				if SFA_Presets[arg].sfaura_xscale2 != nil then
					SFA_XScale2(player, tostring(SFA_Presets[arg].sfaura_xscale2))
				end
				if SFA_Presets[arg].sfaura_yscale2 != nil then
					SFA_YScale2(player, tostring(SFA_Presets[arg].sfaura_yscale2))
				end
				if SFA_Presets[arg].sfaura_opacity != nil then
					SFA_Opacity(player, tostring(SFA_Presets[arg].sfaura_opacity))
				end
				if SFA_Presets[arg].sfaura_opacity2 != nil then
					SFA_Opacity2(player, tostring(SFA_Presets[arg].sfaura_opacity2))
				end
				if SFA_Presets[arg].sfaura_color != nil then
					SFA_Color(player, tostring(SFA_Presets[arg].sfaura_color))
				end
				if SFA_Presets[arg].sfaura_color2 != nil then
					SFA_Color2(player, tostring(SFA_Presets[arg].sfaura_color2))
				end
				if SFA_Presets[arg].sfaura_sprite != nil then
					SFA_Sprite(player, tostring(SFA_Presets[arg].sfaura_sprite))
				end
				if SFA_Presets[arg].sfaura_sprite2 != nil then
					SFA_Sprite2(player, tostring(SFA_Presets[arg].sfaura_sprite2))
				end
				if SFA_Presets[arg].sfaura_soundscheme != nil then
					SFA_SoundScheme(player, tostring(SFA_Presets[arg].sfaura_soundscheme))
				end
				if SFA_Presets[arg].sfaura_draworder != nil then
					SFA_DrawOrder(player, tostring(SFA_Presets[arg].sfaura_draworder))
				end
				if SFA_Presets[arg].sfaura_draworder2 != nil then
					SFA_DrawOrder2(player, tostring(SFA_Presets[arg].sfaura_draworder2))
				end
				if SFA_Presets[arg].sfaura_blendmode != nil then
					SFA_BlendMode(player, tostring(SFA_Presets[arg].sfaura_blendmode))
				end
				if SFA_Presets[arg].sfaura_blendmode2 != nil then
					SFA_BlendMode2(player, tostring(SFA_Presets[arg].sfaura_blendmode2))
				end
				if SFA_Presets[arg].sfaura_outline != nil then
					SFA_Outline(player, tostring(SFA_Presets[arg].sfaura_outline))
				end
				if SFA_Presets[arg].sfaura_outlinecolor != nil then
					SFA_OutlineColor(player, tostring(SFA_Presets[arg].sfaura_outlinecolor))
				end
				if SFA_Presets[arg].sfaura_outlineblend != nil then
					SFA_OutlineBlend(player, tostring(SFA_Presets[arg].sfaura_outlineblend))
				end
				CONS_Printf(player, "Preset \""..SFA_Presets[arg].name.."\" loaded!")
			elseif arg == "list"
				for k, v in pairs(SFA_Presets)
					CONS_Printf(player, k.." - "..SFA_Presets[k].name)
				end
			else
				CONS_Printf(player, "The specified argument is invalid.")
			end
		else
			CONS_Printf(player, "sfaura_preset <id>: Changes all of your aura's settings from a preset.")
			CONS_Printf(player, "Type \"sfaura_preset list\" to see all the available IDs for presets.")
		end
	else
		CONS_Printf(player, "You must be in a map to use this command.")
	end
end)

//I/O stuff
LoadCVar("sfaura_sounds", "client/SuperFormAuras/sfaura_sounds.dat")

addHook("GameQuit", function()
	SaveCVar("sfaura_sounds", "client/SuperFormAuras/sfaura_sounds.dat")
end)

local function LoadSFAConfig(player)
	local file
	local conf
	
	file = io.openlocal("client/SuperFormAuras/sfaura_xscale.dat")
	if file
		conf = file:read("*a")
		SFA_XScale(player, conf)
		file:close()
	else
		SFA_XScale(player, "1")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_yscale.dat")
	if file
		conf = file:read("*a")
		SFA_YScale(player, conf)
		file:close()
	else
		SFA_YScale(player, "1")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_xscale2.dat")
	if file
		conf = file:read("*a")
		SFA_XScale2(player, conf)
		file:close()
	else
		SFA_XScale2(player, "1")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_yscale2.dat")
	if file
		conf = file:read("*a")
		SFA_YScale2(player, conf)
		file:close()
	else
		SFA_YScale2(player, "1")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_conditions.dat")
	if file
		conf = file:read("*a")
		SFA_Conditions(player, conf)
		file:close()
	else
		SFA_Conditions(player, "super")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_color.dat")
	if file
		conf = file:read("*a")
		SFA_Color(player, conf)
		file:close()
	else
		SFA_Color(player, "player")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_color2.dat")
	if file
		conf = file:read("*a")
		SFA_Color2(player, conf)
		file:close()
	else
		SFA_Color2(player, "player")
	end
	
	changesoundsch = false
	file = io.openlocal("client/SuperFormAuras/sfaura_soundscheme.dat")
	if file
		conf = file:read("*a")
		SFA_SoundScheme(player, conf)
		file:close()
	else
		SFA_SoundScheme(player, "ssj")
	end
	changesoundsch = true
	
	file = io.openlocal("client/SuperFormAuras/sfaura_sprite.dat")
	if file
		conf = file:read("*a")
		SFA_Sprite(player, conf)
		file:close()
	else
		SFA_Sprite(player, "ssj")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_sprite2.dat")
	if file
		conf = file:read("*a")
		SFA_Sprite2(player, conf)
		file:close()
	else
		SFA_Sprite2(player, "ssj")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_opacity.dat")
	if file
		conf = file:read("*a")
		SFA_Opacity(player, conf)
		file:close()
	else
		SFA_Opacity(player, "10")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_opacity2.dat")
	if file
		conf = file:read("*a")
		SFA_Opacity2(player, conf)
		file:close()
	else
		SFA_Opacity2(player, "0")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_draworder.dat")
	if file
		conf = file:read("*a")
		SFA_DrawOrder(player, conf)
		file:close()
	else
		SFA_DrawOrder(player, "behind")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_draworder2.dat")
	if file
		conf = file:read("*a")
		SFA_DrawOrder2(player, conf)
		file:close()
	else
		SFA_DrawOrder2(player, "behind")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_blendmode.dat")
	if file
		conf = file:read("*a")
		SFA_BlendMode(player, conf)
		file:close()
	else
		SFA_BlendMode(player, "normal")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_blendmode2.dat")
	if file
		conf = file:read("*a")
		SFA_BlendMode2(player, conf)
		file:close()
	else
		SFA_BlendMode2(player, "normal")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_outline.dat")
	if file
		conf = file:read("*a")
		SFA_Outline(player, conf)
		file:close()
	else
		SFA_Outline(player, "off")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_outlinecolor.dat")
	if file
		conf = file:read("*a")
		SFA_OutlineColor(player, conf)
		file:close()
	else
		SFA_OutlineColor(player, "player")
	end
	
	file = io.openlocal("client/SuperFormAuras/sfaura_outlineblend.dat")
	if file
		conf = file:read("*a")
		SFA_OutlineBlend(player, conf)
		file:close()
	else
		SFA_OutlineBlend(player, "add")
	end
end

addHook("ThinkFrame", do
	if (consoleplayer and consoleplayer.valid)
	and not (configplayer and configplayer.valid)
		configplayer = consoleplayer
		LoadSFAConfig(consoleplayer)
	end
end)

//Function to make a mobj's sprite have a glowing outline
//Must be run every tic
local function SpawnOutlineGlow(mo, color)
	local ol1 = P_SpawnMobj(mo.x, mo.y, mo.z, MT_OVERLAY)
	local ol2 = P_SpawnMobj(mo.x, mo.y, mo.z, MT_OVERLAY)
	local ol3 = P_SpawnMobj(mo.x, mo.y, mo.z, MT_OVERLAY)
	local ol4 = P_SpawnMobj(mo.x, mo.y, mo.z, MT_OVERLAY)
	
	ol1.target = mo
	ol2.target = mo
	ol3.target = mo
	ol4.target = mo
	
	ol1.state = S_GLOWOL
	ol2.state = S_GLOWOL
	ol3.state = S_GLOWOL
	ol4.state = S_GLOWOL
	
	ol1.skin = mo.skin
	ol2.skin = mo.skin
	ol3.skin = mo.skin
	ol4.skin = mo.skin
	
	ol1.color = color
	ol2.color = color
	ol3.color = color
	ol4.color = color
	
	ol1.sprite = mo.sprite
	ol2.sprite = mo.sprite
	ol3.sprite = mo.sprite
	ol4.sprite = mo.sprite
	
	ol1.sprite2 = mo.sprite2
	ol2.sprite2 = mo.sprite2
	ol3.sprite2 = mo.sprite2
	ol4.sprite2 = mo.sprite2
	
	ol1.rollangle = mo.rollangle
	ol2.rollangle = mo.rollangle
	ol3.rollangle = mo.rollangle
	ol4.rollangle = mo.rollangle
	
	ol1.fuse = 2
	ol2.fuse = 2
	ol3.fuse = 2
	ol4.fuse = 2
	
	ol1.colorized = true
	ol2.colorized = true
	ol3.colorized = true
	ol4.colorized = true
	
	ol1.scale = mo.scale
	ol2.scale = mo.scale
	ol3.scale = mo.scale
	ol4.scale = mo.scale
	
	ol1.spritexoffset = mo.spritexoffset+FRACUNIT
	ol2.spritexoffset = mo.spritexoffset-FRACUNIT
	ol3.spriteyoffset = mo.spriteyoffset+FRACUNIT
	ol4.spriteyoffset = mo.spriteyoffset-FRACUNIT
end

//Glowing outline thinker
addHook("MobjThinker", function(mo)
	if mo and mo.valid
		if mo.target and mo.target.valid
		and mo.state == S_GLOWOL
			if mo.target.skin then
				mo.skin = mo.target.skin
			end
			mo.sprite = mo.target.sprite
			mo.sprite2 = mo.target.sprite2
			mo.frame = mo.target.frame
			mo.blendmode = mo.target.player.outlineblend
		end
	end
end, MT_OVERLAY)

addHook("PlayerThink", function(player)
	if not (player.mo and player.mo.valid) then return end
	
	//Conditions to trigger the aura
	player.cantriggeraura = player.playerstate == PST_LIVE and ((player.auraconditions == 1 and (player.powers[pw_super] or player.nightstime)) or player.auraconditions == 2)
	
	if (player.mo.state != S_PLAY_SUPER_TRANS1
	and player.mo.state != S_PLAY_SUPER_TRANS2
	and player.mo.state != S_PLAY_NIGHTS_TRANS1
	and player.mo.state != S_PLAY_NIGHTS_TRANS2)
	and player.cantriggeraura
	and not (player.auramo and player.auramo.valid) then
	
		//Spawn aura mobj if the player is currently in their Super Form
		player.auramo = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_OVERLAY)
		if CV_FindVar("sfaura_sounds").value == 1
		and CV_FindVar("sfaura_allowsounds").value == 1 then
			S_StartSound(player.auramo, SFA_SoundSchemes[player.aurasoundsch].transform)
		end
		player.auramo.target = player.mo
		player.auramo.state = S_SUPERAURA
		if player.auraopacity == 0 then
			player.auramo.flags2 = $|MF2_DONTDRAW
		elseif player.auraopacity > 0 and player.auraopacity < 10
			player.auramo.flags2 = $ & ~MF2_DONTDRAW
			player.auramo.frame = ($ & ~FF_TRANSMASK)|((10-player.auraopacity)<<FF_TRANSSHIFT)
		end
		player.auramo.sprite = SFA_AuraStyles[player.aurastyle].sprite
	end
	
	//Second aura
	if (player.auramo and player.auramo.valid)
	and not (player.auramo2 and player.auramo2.valid)
	and player.auraopacity2 != 0 then
		player.auramo2 = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_OVERLAY)
		player.auramo2.target = player.mo
		player.auramo2.state = S_SUPERAURA2
		if player.auraopacity2 == 0 then
			player.auramo2.flags2 = $|MF2_DONTDRAW
		elseif player.auraopacity2 > 0 and player.auraopacity2 < 10
			player.auramo2.flags2 = $ & ~MF2_DONTDRAW
			player.auramo2.frame = ($ & ~FF_TRANSMASK)|((10-player.auraopacity2)<<FF_TRANSSHIFT)
		end
		player.auramo2.sprite = SFA_AuraStyles[player.aurastyle2].sprite
	end
	
	//Aura overlay
	if (SFA_AuraStyles[player.aurastyle]
	and SFA_AuraStyles[player.aurastyle].overlay != nil
	and SFA_AuraStyles[player.aurastyle].overlay != SPR_NULL)
	and (player.mo.state != S_PLAY_SUPER_TRANS1
	and player.mo.state != S_PLAY_SUPER_TRANS2
	and player.mo.state != S_PLAY_NIGHTS_TRANS1
	and player.mo.state != S_PLAY_NIGHTS_TRANS2)
	and player.cantriggeraura
	and not (player.auraovmo and player.auraovmo.valid) then
		player.auraovmo = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_OVERLAY)
		player.auraovmo.target = player.auramo
		player.auraovmo.tracer = player.mo
		player.auraovmo.state = S_AURAOVERLAY
	end
	if (player.auraovmo and player.auraovmo.valid)
	and SFA_AuraStyles[player.aurastyle].overlay != nil then
		player.auraovmo.sprite = SFA_AuraStyles[player.aurastyle].overlay
	end
	
	if (SFA_AuraStyles[player.aurastyle] == nil
	or SFA_AuraStyles[player.aurastyle].overlay == nil
	or SFA_AuraStyles[player.aurastyle].overlay == SPR_NULL)
	and (player.auraovmo and player.auraovmo.valid) then
		P_RemoveMobj(player.auraovmo)
	end
	
	//Second aura overlay
	if (SFA_AuraStyles[player.aurastyle2]
	and SFA_AuraStyles[player.aurastyle2].overlay != nil
	and SFA_AuraStyles[player.aurastyle2].overlay != SPR_NULL)
	and (player.auramo2 and player.auramo2.valid)
	and not (player.auraovmo2 and player.auraovmo2.valid) then
		player.auraovmo2 = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_OVERLAY)
		player.auraovmo2.target = player.auramo2
		player.auraovmo2.tracer = player.mo
		player.auraovmo2.state = S_AURAOVERLAY2
	end
	if (player.auraovmo2 and player.auraovmo2.valid)
	and SFA_AuraStyles[player.aurastyle2].overlay != nil then
		player.auraovmo2.sprite = SFA_AuraStyles[player.aurastyle2].overlay
	end
	
	if (SFA_AuraStyles[player.aurastyle2] == nil
	or SFA_AuraStyles[player.aurastyle2].overlay == nil
	or SFA_AuraStyles[player.aurastyle2].overlay == SPR_NULL)
	and (player.auraovmo2 and player.auraovmo2.valid) then
		P_RemoveMobj(player.auraovmo2)
	end
	
	if (player.auramo and player.auramo.valid)
	and player.auraconditions == 2 then
		player.mo.renderflags = $|RF_FULLBRIGHT
	else
		player.mo.renderflags = $ & ~RF_FULLBRIGHT
	end
	
	if not player.cantriggeraura
	and (player.auramo and player.auramo.valid) then
		//Remove aura if player is no longer in their Super Form
		if CV_FindVar("sfaura_sounds").value == 1 
		and CV_FindVar("sfaura_allowsounds").value == 1 then
			S_StopSoundByID(player.auramo, SFA_SoundSchemes[player.aurasoundsch].passive)
			S_StartSound(player.mo, SFA_SoundSchemes[player.aurasoundsch].detransform)
		end
		P_RemoveMobj(player.auramo)
		if (player.auraovmo and player.auraovmo.valid) then
			P_RemoveMobj(player.auraovmo)
		end
	end
	
	if (player.auramo2 and player.auramo2.valid)
	and (not (player.auramo and player.auramo.valid)
	or player.auraopacity2 == 0) then
		P_RemoveMobj(player.auramo2)
	end
	
	//Glowing outline
	if (player.mo.state != S_PLAY_SUPER_TRANS1
	and player.mo.state != S_PLAY_SUPER_TRANS2
	and player.mo.state != S_PLAY_NIGHTS_TRANS1
	and player.mo.state != S_PLAY_NIGHTS_TRANS2)
	and player.glowoutline
	and player.cantriggeraura then
		if player.outlinecolor == "player" then
			SpawnOutlineGlow(player.mo, player.mo.color)
		elseif player.outlinecolor == "opposite" then
			SpawnOutlineGlow(player.mo, ColorOpposite(player.mo.color))
		elseif type(player.outlinecolor) == "number" then
			SpawnOutlineGlow(player.mo, player.outlinecolor)
		end
	end
end)

//Thinker for aura
addHook("MobjThinker", function(aura)
	if aura.state != S_SUPERAURA then return end
	
	//Timer
	if aura.passfxtimer == nil then
		aura.passfxtimer = 0
	end
	aura.passfxtimer = $+1
	
	//Aura frame cycle behavior
	if aura.nextframe == nil then
		aura.nextframe = 0
	end
	aura.frame = aura.nextframe|($ & ~FF_FRAMEMASK)
	if (leveltime % SFA_AuraStyles[aura.target.player.aurastyle].tics == 1) then
		aura.nextframe = $+1
	end
	if aura.nextframe > SFA_AuraStyles[aura.target.player.aurastyle].frames-1 then
		aura.nextframe = 0
	end
	
	//Aura properties
	
	//Aura colors based on current supercolor:
		//Silver: Bone (2)
		//Red: Super Red 5 (122)
		//Orange: Apricot (43)
		//Gold: Yellow (51)
		//Peridot: Peridot (56)
		//Sky: Sky (77)
		//Purple: Super Purple 4 (146)
		//Rust: Copper (42)
		//Tan: Super Tan 4 (156)
	if aura.target.player.auracolor == "player" then
		if aura.target.color >= 113 and aura.target.color <= 117 then
			aura.color = SKINCOLOR_BONE
		elseif aura.target.color >= 118 and aura.target.color <= 122 then
			aura.color = SKINCOLOR_SUPERRED5
		elseif aura.target.color >= 123 and aura.target.color <= 127 then
			aura.color = SKINCOLOR_APRICOT
		elseif aura.target.color >= 128 and aura.target.color <= 132 then
			aura.color = SKINCOLOR_YELLOW
		elseif aura.target.color >= 133 and aura.target.color <= 137 then
			aura.color = SKINCOLOR_PERIDOT
		elseif aura.target.color >= 138 and aura.target.color <= 142 then
			aura.color = SKINCOLOR_SKY
		elseif aura.target.color >= 143 and aura.target.color <= 147 then
			aura.color = SKINCOLOR_SUPERPURPLE4
		elseif aura.target.color >= 148 and aura.target.color <= 152 then
			aura.color = SKINCOLOR_COPPER
		elseif aura.target.color >= 153 and aura.target.color <= 157 then
			aura.color = SKINCOLOR_SUPERTAN4
		else
			aura.color = aura.target.color
		end
	elseif aura.target.player.auracolor == "opposite" then
		aura.color = ColorOpposite(aura.target.color)
	elseif type(aura.target.player.auracolor) == "number" then
		aura.color = aura.target.player.auracolor
	end
	
	if SFA_AuraStyles[aura.target.player.aurastyle].scale != nil then
		aura.spritexscale = FixedMul(aura.target.player.auraxscale, SFA_AuraStyles[aura.target.player.aurastyle].scale)
		aura.spriteyscale = FixedMul(aura.target.player.aurayscale, SFA_AuraStyles[aura.target.player.aurastyle].scale)
	else
		aura.spritexscale = aura.target.player.auraxscale
		aura.spriteyscale = aura.target.player.aurayscale
	end
	
	states[aura.state].var1 = aura.target.player.auradraworder
	
	//Play the aura's passive sound
	if (aura.passfxtimer % SFA_SoundSchemes[aura.target.player.aurasoundsch].passivetics == 1)
	and CV_FindVar("sfaura_sounds").value == 1
	and CV_FindVar("sfaura_allowsounds").value == 1 then
		S_StartSound(aura, SFA_SoundSchemes[aura.target.player.aurasoundsch].passive)
	end
	
	aura.blendmode = aura.target.player.aurablend
end, MT_OVERLAY)

//Thinker for second aura
addHook("MobjThinker", function(aura)
	if aura.state != S_SUPERAURA2 then return end
	
	//Aura frame cycle behavior
	if aura.nextframe == nil then
		aura.nextframe = 0
	end
	aura.frame = aura.nextframe|($ & ~FF_FRAMEMASK)
	if (leveltime % SFA_AuraStyles[aura.target.player.aurastyle2].tics == 1) then
		aura.nextframe = $+1
	end
	if aura.nextframe > SFA_AuraStyles[aura.target.player.aurastyle2].frames-1 then
		aura.nextframe = 0
	end
	
	//Aura properties
	if aura.target.player.auracolor2 == "player" then
		if aura.target.color >= 113 and aura.target.color <= 117 then
			aura.color = SKINCOLOR_BONE
		elseif aura.target.color >= 118 and aura.target.color <= 122 then
			aura.color = SKINCOLOR_SUPERRED5
		elseif aura.target.color >= 123 and aura.target.color <= 127 then
			aura.color = SKINCOLOR_APRICOT
		elseif aura.target.color >= 128 and aura.target.color <= 132 then
			aura.color = SKINCOLOR_YELLOW
		elseif aura.target.color >= 133 and aura.target.color <= 137 then
			aura.color = SKINCOLOR_PERIDOT
		elseif aura.target.color >= 138 and aura.target.color <= 142 then
			aura.color = SKINCOLOR_SKY
		elseif aura.target.color >= 143 and aura.target.color <= 147 then
			aura.color = SKINCOLOR_SUPERPURPLE4
		elseif aura.target.color >= 148 and aura.target.color <= 152 then
			aura.color = SKINCOLOR_COPPER
		elseif aura.target.color >= 153 and aura.target.color <= 157 then
			aura.color = SKINCOLOR_SUPERTAN4
		else
			aura.color = aura.target.color
		end
	elseif aura.target.player.auracolor2 == "opposite" then
		aura.color = ColorOpposite(aura.target.color)
	elseif type(aura.target.player.auracolor2) == "number" then
		aura.color = aura.target.player.auracolor2
	end
	
	if SFA_AuraStyles[aura.target.player.aurastyle].scale != nil then
		aura.spritexscale = FixedMul(aura.target.player.auraxscale2, SFA_AuraStyles[aura.target.player.aurastyle].scale)
		aura.spriteyscale = FixedMul(aura.target.player.aurayscale2, SFA_AuraStyles[aura.target.player.aurastyle].scale)
	else
		aura.spritexscale = aura.target.player.auraxscale2
		aura.spriteyscale = aura.target.player.aurayscale2
	end
	
	states[aura.state].var1 = aura.target.player.auradraworder2
	
	aura.blendmode = aura.target.player.aurablend2
end, MT_OVERLAY)

//Thinker for aura overlay
addHook("MobjThinker", function(aura)
	if aura.state != S_AURAOVERLAY then return end
	
	//Aura frame cycle behavior
	if aura.nextframe == nil then
		aura.nextframe = 0
	end
	aura.frame = aura.nextframe|($ & ~FF_FRAMEMASK)
	if SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_opacity != nil then
		aura.frame = ($ & ~FF_TRANSMASK)|SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_opacity
	else
		aura.frame = ($ & ~FF_TRANSMASK)
	end
	if (leveltime % SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_tics == 1) then
		aura.nextframe = $+1
	end
	if aura.nextframe > SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_frames-1 then
		aura.nextframe = 0
	end

	if SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_scale then
		aura.spritexscale = FixedMul(aura.tracer.player.auraxscale, SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_scale)
		aura.spriteyscale = FixedMul(aura.tracer.player.aurayscale, SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_scale)
	else
		aura.spritexscale = aura.tracer.player.auraxscale
		aura.spriteyscale = aura.tracer.player.aurayscale
	end
	
	if SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_blending != nil then
		aura.blendmode = SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_blending
	else
		aura.blendmode = AST_TRANSLUCENT
	end
end, MT_OVERLAY)

//Thinker for second aura overlay
addHook("MobjThinker", function(aura)
	if aura.state != S_AURAOVERLAY2 then return end
	
	//Aura frame cycle behavior
	if aura.nextframe == nil then
		aura.nextframe = 0
	end
	aura.frame = aura.nextframe|($ & ~FF_FRAMEMASK)
	if SFA_AuraStyles[aura.tracer.player.aurastyle2].overlay_opacity != nil then
		aura.frame = ($ & ~FF_TRANSMASK)|SFA_AuraStyles[aura.tracer.player.aurastyle2].overlay_opacity
	else
		aura.frame = ($ & ~FF_TRANSMASK)
	end
	if (leveltime % SFA_AuraStyles[aura.tracer.player.aurastyle2].overlay_tics == 1) then
		aura.nextframe = $+1
	end
	if aura.nextframe > SFA_AuraStyles[aura.tracer.player.aurastyle2].overlay_frames-1 then
		aura.nextframe = 0
	end

	if SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_scale then
		aura.spritexscale = FixedMul(aura.tracer.player.auraxscale2, SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_scale)
		aura.spriteyscale = FixedMul(aura.tracer.player.aurayscale2, SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_scale)
	else
		aura.spritexscale = aura.tracer.player.auraxscale2
		aura.spriteyscale = aura.tracer.player.aurayscale2
	end
	
	if SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_blending != nil then
		aura.blendmode = SFA_AuraStyles[aura.tracer.player.aurastyle].overlay_blending
	else
		aura.blendmode = AST_TRANSLUCENT
	end
end, MT_OVERLAY)